@@ -6,6 +6,8 @@ import Imports
 import qualified Number
 import qualified Number.F2m
 import qualified BCrypt
+import qualified ECC
+import qualified ECC.Edwards25519
 import qualified Hash
 import qualified Poly1305
 import qualified Salsa
@@ -21,12 +23,14 @@ import qualified KAT_PBKDF2
 import qualified KAT_Curve25519
 import qualified KAT_Curve448
 import qualified KAT_Ed25519
+import qualified KAT_Ed448
 import qualified KAT_OTP
 import qualified KAT_PubKey
 import qualified KAT_Scrypt
 -- symmetric cipher --------------------
 import qualified KAT_AES
 import qualified KAT_Blowfish
+import qualified KAT_CAST5
 import qualified KAT_Camellia
 import qualified KAT_DES
 import qualified KAT_RC4
@@ -52,6 +56,7 @@ tests = testGroup "cryptonite"
     , KAT_Curve25519.tests
     , KAT_Curve448.tests
     , KAT_Ed25519.tests
+    , KAT_Ed448.tests
     , KAT_PubKey.tests
     , KAT_OTP.tests
     , testGroup "KDF"
@@ -64,6 +69,7 @@ tests = testGroup "cryptonite"
     , testGroup "block-cipher"
         [ KAT_AES.tests
         , KAT_Blowfish.tests
+        , KAT_CAST5.tests
         , KAT_Camellia.tests
         , KAT_DES.tests
         , KAT_TripleDES.tests
@@ -77,6 +83,8 @@ tests = testGroup "cryptonite"
         , XSalsa.tests
         ]
     , KAT_AFIS.tests
+    , ECC.tests
+    , ECC.Edwards25519.tests
     ]
 
 main = defaultMain tests
